﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace MoHooSpore
{
    public sealed class CRC
    {
        private static ushort[] crc16Table = null;
        private static uint[] crc32Table = null;

        private static void MakeCRC16Table()
        {
            if (crc16Table != null)
            {
                return;
            }
            crc16Table = new ushort[256];
            for (ushort i = 0; i < 256; i++)
            {
                ushort vCRC = i;
                for (int j = 0; j < 8; j++)
                {
                    if (vCRC % 2 == 0)
                    {
                        vCRC = (ushort)(vCRC >> 1);
                    }
                    else
                    {
                        vCRC = (ushort)((vCRC >> 1) ^ 0x8408);
                    }
                }
                crc16Table[i] = vCRC;
            }
        }
        //设计 ZswangY37 2007-02-14
        private static void MakeCRC32Table()
        {
            if (crc32Table != null)
            {
                return;
            }
            crc32Table = new uint[256];
            for (uint i = 0; i < 256; i++)
            {
                uint vCRC = i;
                for (int j = 0; j < 8; j++)
                {
                    if (vCRC % 2 == 0)
                    {
                        vCRC = (uint)(vCRC >> 1);
                    }
                    else
                    {
                        vCRC = (uint)((vCRC >> 1) ^ 0xEDB88320);
                    }
                }
                crc32Table[i] = vCRC;
            }
        }

        public static ushort UpdateCRC16(byte @byte, ushort seed)
        {
            return (ushort)(crc16Table[(seed & 0x000000FF) ^ @byte] ^ (seed >> 8));
        }

        public static uint UpdateCRC32(byte @byte, uint seed)
        {
            return (uint)(crc32Table[(seed & 0x000000FF) ^ @byte] ^ (seed >> 8));
        }

        public static ushort CRC16(byte[] bytes)
        {
            MakeCRC16Table();
            ushort result = 0xFFFF;
            foreach (byte vByte in bytes)
            {
                result = UpdateCRC16(vByte, result);
            }
            return (ushort)(~result);
        }

        public static ushort CRC16(string str, Encoding encoding)
        {
            return CRC16(encoding.GetBytes(str));
        }

        public static ushort CRC16(string str)
        {
            return CRC16(str, Encoding.UTF8);
        }

        public static uint CRC32(byte[] bytes)
        {
            MakeCRC32Table();
            uint result = 0xFFFFFFFF;
            foreach (byte vByte in bytes)
            {
                result = UpdateCRC32(vByte, result);
            }
            return (uint)(~result);
        }
        public static uint CRC32(string str, Encoding encoding)
        {
            return CRC32(encoding.GetBytes(str));
        }
        public static uint CRC32(string str)
        {
            return CRC32(str, Encoding.UTF8);
        }
        public static uint CRC32(Stream stream, long start, long length)
        {
            MakeCRC32Table();
            uint result = 0xFFFFFFFF;

            long end = start + length;
            var bytes = new byte[8192];
            var readed = 0;
            stream.Seek(start, SeekOrigin.Begin);
            var count = bytes.Length;
            while (stream.Position < end)
            {
                if (end - stream.Position < bytes.Length)
                {
                    count = (int)(end - stream.Position);
                }
                readed = stream.Read(bytes, 0, count);
                if (readed > 0)
                {
                    for (var i = 0; i < readed; i++)
                    {
                        result = UpdateCRC32(bytes[i], result);
                    }
                }
            }

            
            return (uint)(~result);
        }

        public static uint CRC32(Stream stream)
        {
            MakeCRC32Table();
            uint result = 0xFFFFFFFF;

            var bytes = new byte[8192];
            var readed = 0;
            stream.Seek(0, SeekOrigin.Begin);
            var count = bytes.Length;
            while (stream.Position < stream.Length)
            {
                readed = stream.Read(bytes, 0, count);
                if (readed > 0)
                {
                    for (var i = 0; i < readed; i++)
                    {
                        result = UpdateCRC32(bytes[i], result);
                    }
                }
            }

            return (uint)(~result);
        }
    }
}
