﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;

namespace MoHooSpore
{
    public class FileReader
    {
        public Stream SourceStream { get; set; }

        public FileReader()
        { }

        public FileReader(Stream stream)
        {
            this.SourceStream = stream;
        }

        public EmbedFile[] Read()
        {
            var infos = this.Analyze();

            if (infos == null || infos.Length == 0)
            {
                return null;
            }

            var blocks = new List<EmbedFile>();
            foreach (var i in infos)
            {
                blocks.Add(new EmbedFile
                {
                    FileName = i.FileName,
                    Stream = new EmbedFileStream(this.SourceStream, i.Offset, i.Length)
                });
            }
            return blocks.ToArray();
        }

        private EmbedFileInfo[] Analyze()
        {
            if (this.SourceStream == null)
            {
                throw new ArgumentNullException("SourceStream");
            }

            // 获取起始点
            var start = this.GetStartIndex();
            Debug.WriteLine("[r]start:" + start);

            // 获取数据长度
            var b1 = new byte[8];
            this.SourceStream.Position = start;
            this.SourceStream.Read(b1, 0, 8);
            var dLen = BitConverter.ToInt64(b1, 0);
            Debug.WriteLine("[r]length:" + dLen);

            // 获取CRC
            var b2 = new byte[4];
            this.SourceStream.Position = start + dLen + 4;
            this.SourceStream.Read(b2, 0, 4);
            var crc = BitConverter.ToUInt32(b2, 0);
            Debug.WriteLine("[r]crc:" + crc);

            // 计算CRC
            var crc2 = CRC.CRC32(this.SourceStream, start, dLen + 4);
            Debug.WriteLine("[r]crc2:" + crc2);

            if (crc == crc2)
            {
                var end = start + dLen + 4;

                var embeds = new List<EmbedFileInfo>();

                var fStart = start + 12;
                while (fStart < end)
                {
                    // 获取文件名长度
                    var b3 = new byte[4];
                    this.SourceStream.Position = fStart;
                    this.SourceStream.Read(b3, 0, 4);
                    var fnLen = BitConverter.ToInt32(b3, 0);
                    Debug.WriteLine("[r]fnLen:" + fnLen);

                    // 获取文件名
                    var b4 = new byte[fnLen];
                    this.SourceStream.Position = fStart + 4;
                    this.SourceStream.Read(b4, 0, b4.Length);
                    var filename = Encoding.Unicode.GetString(b4);
                    Debug.WriteLine("[r]filename:" + filename);

                    // 获取文件长度
                    var b5 = new byte[8];
                    this.SourceStream.Read(b5, 0, 8);
                    var fLen = BitConverter.ToInt64(b5, 0);
                    Debug.WriteLine("[r]fLen:" + fLen);

                    var fOffset = this.SourceStream.Position;

                    // 获取CRC
                    var b6 = new byte[4];
                    this.SourceStream.Position += fLen;
                    this.SourceStream.Read(b6, 0, 4);
                    var fCrc = BitConverter.ToInt32(b6, 0);
                    Debug.WriteLine("[r]fCrc:" + fCrc);

                    var f = new EmbedFileInfo
                    {
                        FileName = filename,
                        Length = fLen,
                        Offset = fOffset
                    };
                    embeds.Add(f);

                    fStart = f.Offset + f.Length + 4;
                }

                return embeds.ToArray();
            }

            return null;
        }

        private long GetStartIndex()
        {
            return IndexOf("IMHD") - 8;
        }

        private long IndexOf(string flag, long start = 0)
        {
            long index = -1L;

            this.SourceStream.Position = start;

            byte[] flags = Encoding.ASCII.GetBytes(flag);
            int read = this.SourceStream.ReadByte();
            int idx = 0;
            while (read != -1)
            {
                for (var i = 0; i < flags.Length; i++)
                {
                    if (read == -1 || read != flags[i])
                    {
                        break;
                    }
                    else
                    {
                        //Console.Write((char)read);

                        read = this.SourceStream.ReadByte();
                        idx++;
                        if (i == flags.Length - 1)
                        {
                            index = idx;
                        }
                    }
                }

                if (index == -1)
                {
                    read = this.SourceStream.ReadByte();
                    idx++;
                }
                else
                {
                    break;
                }
            }

            return index + start - flags.Length;
        }

    }
}
