﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;

namespace MoHooSpore
{
    public class FileWriter
    {
        public Stream SourceStream { get; set; }

        public FileWriter()
        { }

        public FileWriter(Stream stream)
        {
            this.SourceStream = stream;
        }

        private byte[] IMHD = new byte[12];
        private byte[] IEND = new byte[12];

        private long start;
        private long length;

        public void Write(string[] filenames)
        {
            this.start = this.SourceStream.Length - IEND.Length;

            // 提取结束标记
            this.SourceStream.Position = this.start;
            this.SourceStream.Read(this.IEND, 0, this.IEND.Length);
            this.SourceStream.Position = this.start;

            // 生成数据块标记
            Array.Copy(Encoding.ASCII.GetBytes("IMHD"), 0, this.IMHD, 8, 4);

            // 写入数据块起始标记
            this.SourceStream.Write(this.IMHD, 0, this.IMHD.Length);
            this.length += this.IMHD.Length;

            // 写入数据内容
            //byte[] h2 = Encoding.Unicode.GetBytes(BOUNDARY_HEADER);
            //this.SourceStream.Write(h2, 0, h2.Length);
            //this.length += h2.Length;
            foreach (var file in filenames)
            {
                if (File.Exists(file))
                {
                    var b1 = Encoding.Unicode.GetBytes(Path.GetFileName(file));
                    var b2 = BitConverter.GetBytes(b1.Length);
                    // 写入文件名长度
                    this.SourceStream.Write(b2, 0, b2.Length);
                    this.length += b2.Length;
                    // 写入文件名
                    this.SourceStream.Write(b1, 0, b1.Length);
                    this.length += b1.Length;

                    using (var fs = new FileStream(file, FileMode.Open))
                    {
                        var b3 = BitConverter.GetBytes(fs.Length);
                        // 写入文件长度
                        this.SourceStream.Write(b3, 0, b3.Length);
                        this.length += b3.Length;

                        // 写入文件内容
                        var buffer = new byte[8192];
                        var count = 0;
                        while (fs.Position < fs.Length)
                        {
                            count = fs.Read(buffer, 0, buffer.Length);
                            this.SourceStream.Write(buffer, 0, count);
                        }
                        this.length += fs.Length;

                        var b4 = BitConverter.GetBytes(CRC.CRC32(fs));
                        // 写入CRC校验
                        this.SourceStream.Write(b4, 0, b4.Length);
                        this.length += b4.Length;
                    }
                }
            }
            
            // 写入数据总长度
            var lenBytes = BitConverter.GetBytes(this.length - 4);
            this.SourceStream.Position = this.start;
            this.SourceStream.Write(lenBytes, 0, lenBytes.Length);
            Debug.WriteLine("[w]length:" + (this.length - 4));

            // 写入数据CRC
            var crc = CRC.CRC32(this.SourceStream, this.start, this.length);
            var crcBytes = BitConverter.GetBytes(crc);
            this.SourceStream.Position = this.start + this.length;
            this.SourceStream.Write(crcBytes, 0, crcBytes.Length);
            Debug.WriteLine("[w]crc:" + crc);

            // 回写结束标记
            this.SourceStream.Write(this.IEND, 0, this.IEND.Length);
        }
    }
}
